from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
from dateutil.relativedelta import relativedelta
from datetime import timedelta


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}

        # response = requests.get('https://www.rochford.gov.uk/online-bin-collections-calendar', headers=headers)
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()
        year = soup.find_all("table", {"class": "responsive-enabled govuk-table"})

        current_month = datetime.now().strftime("%B %Y")
        next_month = (datetime.now() + relativedelta(months=1, day=1)).strftime("%B %Y")

        for month in year:
            heading = (
                month.find("th", {"class": "govuk-table__header"}).get_text().strip()
            )
            if heading == current_month or heading == next_month:
                for week in month.find("tbody").find_all(
                    "tr", {"class": "govuk-table__row"}
                ):
                    week_text = week.get_text().strip().split("\n")
                    collection_date = datetime.strptime(
                        remove_ordinal_indicator_from_date_string(
                            week_text[0].split(" - ")[0]
                        )
                        .strip(),
                        "%A %d %B",
                    )
                    next_collection = collection_date.replace(year=datetime.now().year)
                    if datetime.now().month == 12 and next_collection.month == 1:
                        next_collection = next_collection + relativedelta(years=1)
                    bin_type = (
                        week_text[1]
                        .replace("collection week", "bin")
                        .strip()
                        .capitalize()
                    )
                    if next_collection.date() >= (datetime.now().date() - timedelta(6)):
                        dict_data = {
                            "type": bin_type,
                            "collectionDate": next_collection.strftime(date_format),
                        }
                        data["bins"].append(dict_data)
            else:
                continue

        return data



from datetime import datetime

import dateutil.parser
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        user_uprn = kwargs.get("uprn")
        web_driver = kwargs.get("web_driver")
        headless = kwargs.get("headless")
        check_uprn(user_uprn)

        root_url = "https://forms.rbwm.gov.uk/bincollections?uprn="
        api_url = root_url + user_uprn

        # Create Selenium webdriver
        driver = create_webdriver(web_driver, headless, None, __name__)
        driver.get(api_url)

        soup = BeautifulSoup(driver.page_source, features="html.parser")
        soup.prettify()

        # Get collections div
        next_collection_div = soup.find("div", {"class": "widget-bin-collections"})

        for tbody in next_collection_div.find_all("tbody"):
            for tr in tbody.find_all("tr"):
                td = tr.find_all("td")
                next_collection_type = td[0].get_text()
                next_collection_date = dateutil.parser.parse(td[1].get_text())
                print(next_collection_date)
                dict_data = {
                    "type": next_collection_type,
                    "collectionDate": next_collection_date.strftime("%d/%m/%Y"),
                }
                data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
from uk_bin_collection.uk_bin_collection.common import *
from datetime import datetime


class CouncilClass(AbstractGetBinDataClass):
    def parse_data(self, page: str, **kwargs) -> dict:
        soup = BeautifulSoup(page.text, features="html.parser")
        data = {"bins": []}

        for bin_section in soup.select('div[class*="containererf"]'):
            date_text = bin_section.find(id="content").text.strip()
            collection_date = datetime.strptime(date_text, "%A, %d/%m/%Y")

            bin_types = bin_section.find(id="main1").findAll("li")
            for bin_type in bin_types:
                bin_type_name = bin_type.text.split("(")[0].strip()

                data["bins"].append(
                    {
                        "type": bin_type_name,
                        "collectionDate": collection_date.strftime(date_format),
                    }
                )

        return data



import time

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_paon(user_paon)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get("https://beta.npt.gov.uk/bins-and-recycling/bin-day-finder/")

            # Accept cookies banner
            cookieAccept = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located((By.ID, "ccc-notify-accept"))
            )
            cookieAccept.click()

            # Populate postcode field
            inputElement_postcode = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.ID,
                        "PostCode",
                    )
                )
            )
            inputElement_postcode.send_keys(user_postcode)

            # Click search button
            findAddress = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.XPATH,
                        "//button[@value='Find address']",
                    )
                )
            )
            findAddress.click()

            # Wait for the 'Select address' dropdown to appear and select option matching UPRN
            dropdown = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.ID,
                        "Address",
                    )
                )
            )
            # Wait for the 'Select address' dropdown to appear and select option matching the house name/number
            WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.XPATH,
                        "//select[@ID='Address']//option[contains(., '"
                        + user_paon
                        + "')]",
                    )
                )
            ).click()

            # Wait for the submit button to appear, then click it to get the collection dates
            submit = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.XPATH,
                        "//input[@value='Find bin day']",
                    )
                )
            )
            submit.click()

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            soup = soup.find(
                "div",
                {"id": "contentInner"},
            )

            # Get the dates
            for date in soup.find_all("h2"):
                if date.get_text(strip=True) != "Bank Holidays":
                    bin_date = datetime.strptime(
                        date.get_text(strip=True)
                        .removesuffix("(Today)")
                        .removesuffix("(Tomorrow)")
                        .replace("&nbsp", " ")
                        + " "
                        + datetime.now().strftime("%Y"),
                        "%A, %d %B %Y",
                    )
                    bin_types_wrapper = date.find_next_sibling("div")
                    for bin_type_wrapper in bin_types_wrapper.find_all(
                        "div",
                        {
                            "class": "card-body ps-5 ps-md-4 ps-lg-5 position-relative bg-white"
                        },
                    ):
                        if bin_date and bin_type_wrapper:
                            bin_type = bin_type_wrapper.find("a").get_text(strip=True)
                            bin_type += (
                                " ("
                                + bin_type_wrapper.find("span").get_text(strip=True)
                                + ")"
                            )
                            dict_data = {
                                "type": bin_type,
                                "collectionDate": bin_date.strftime(date_format),
                            }
                            data["bins"].append(dict_data)

            data["bins"].sort(
                key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
            )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from datetime import datetime

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Get and check UPRN
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        user_uprn = user_uprn.zfill(
            12
        )  # Wigan is expecting 12 character UPRN or else it falls over, expects 0 padded UPRNS at the start for any that aren't 12 chars

        user_postcode = kwargs.get("postcode")
        check_postcode(user_postcode)

        # Start a new session to walk through the form
        requests.packages.urllib3.disable_warnings()
        s = requests.Session()

        # Get our initial session running
        response = s.get("https://apps.wigan.gov.uk/MyNeighbourhood/")

        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        # Grab the ASP variables needed to continue
        payload = {
            "__VIEWSTATE": (soup.find("input", {"id": "__VIEWSTATE"}).get("value")),
            "__VIEWSTATEGENERATOR": (
                soup.find("input", {"id": "__VIEWSTATEGENERATOR"}).get("value")
            ),
            "__EVENTVALIDATION": (
                soup.find("input", {"id": "__EVENTVALIDATION"}).get("value")
            ),
            "ctl00$ContentPlaceHolder1$txtPostcode": (user_postcode),
            "ctl00$ContentPlaceHolder1$btnPostcodeSearch": ("Search"),
        }

        # Use the above to get to the next page with address selection
        response = s.post("https://apps.wigan.gov.uk/MyNeighbourhood/", payload)

        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        # Load the new variables that are constant and can't be gotten from the page
        payload = {
            "__EVENTTARGET": ("ctl00$ContentPlaceHolder1$lstAddresses"),
            "__EVENTARGUMENT": (""),
            "__LASTFOCUS": (""),
            "__VIEWSTATE": (soup.find("input", {"id": "__VIEWSTATE"}).get("value")),
            "__VIEWSTATEGENERATOR": (
                soup.find("input", {"id": "__VIEWSTATEGENERATOR"}).get("value")
            ),
            "__EVENTVALIDATION": (
                soup.find("input", {"id": "__EVENTVALIDATION"}).get("value")
            ),
            "ctl00$ContentPlaceHolder1$txtPostcode": (user_postcode),
            "ctl00$ContentPlaceHolder1$lstAddresses": ("UPRN" + user_uprn),
        }

        # Get the final page with the actual dates
        response = s.post("https://apps.wigan.gov.uk/MyNeighbourhood/", payload)

        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        # Get the dates.
        for bins in soup.find_all("div", {"class": "BinsRecycling"}):
            bin_type = bins.find("h2").text
            binCollection = bins.find("div", {"class": "dateWrapper-next"}).get_text(
                strip=True
            )
            binData = datetime.strptime(
                re.sub(r"(\d)(st|nd|rd|th)", r"\1", binCollection), "%A%d%b%Y"
            )
            if binData:
                dict_data = {
                    "type": bin_type,
                    "collectionDate": binData.strftime(date_format),
                }
                data["bins"].append(dict_data)

        return data



import time
from datetime import datetime

from selenium.webdriver.support.ui import Select
from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    # Extract data from the table
    def format_date(self, date_str):
        # Convert date format from "Fri 31 May 2024" to "31/05/2024"
        date_match = re.search(r"\d{1,2} \w+ \d{4}", date_str)
        if date_match:
            date_obj = re.search(r"(\d{1,2}) (\w+) (\d{4})", date_match.group(0))
            day = date_obj.group(1).zfill(2)
            month_name = date_obj.group(2)
            month = {
                "January": "01",
                "February": "02",
                "March": "03",
                "April": "04",
                "May": "05",
                "June": "06",
                "July": "07",
                "August": "08",
                "September": "09",
                "October": "10",
                "November": "11",
                "December": "12",
            }[month_name]
            year = date_obj.group(3)
            formatted_date = f"{day}/{month}/{year}"
        else:
            formatted_date = "Unknown Date"
        return formatted_date

    def extract_bin_data(self, article):
        date = article.find("div", class_="binday__cell--day").text.strip()
        bin_type_class = article.get("class")[
            1
        ]  # Assuming the second class indicates the bin type
        bin_type = "black" if "black" in bin_type_class else "silver"
        formatted_date = self.format_date(date)
        return {"type": bin_type, "collectionDate": formatted_date}

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            # Make a BS4 object

            page = "https://pre.southkesteven.gov.uk/BinSearch.aspx"

            user_postcode = kwargs.get("postcode")
            user_uprn = kwargs.get("uprn")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            house_number = kwargs.get("paon")

            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            wait = WebDriverWait(driver, 60)

            inputElement_postcodesearch = wait.until(
                EC.visibility_of_element_located((By.ID, "title"))
            )
            inputElement_postcodesearch.clear()

            inputElement_postcodesearch.send_keys(user_postcode)

            inputElement_postcodesearch_btn = wait.until(
                EC.element_to_be_clickable((By.XPATH, "//button/span[text()='Search']"))
            )
            inputElement_postcodesearch_btn.click()

            inputElement_select_address = wait.until(
                EC.element_to_be_clickable((By.ID, "address"))
            )

            # Now create a Select object based on the found element
            dropdown = Select(inputElement_select_address)

            # Select the option by visible text
            dropdown.select_by_visible_text(house_number)

            inputElement_results_btn = wait.until(
                EC.element_to_be_clickable(
                    (By.XPATH, "//button[text()='View your bin days']")
                )
            )
            inputElement_results_btn.click()

            p_element = wait.until(
                EC.presence_of_element_located(
                    (
                        By.XPATH,
                        "//p[contains(text(), 'Your next bin collection date is ')]",
                    )
                )
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")
            soup.prettify()

            bin_data = []

            # Extract data from the first aside element
            first_aside = soup.find("aside", class_="alert")
            if first_aside:
                next_collection_date = first_aside.find(
                    "span", class_="alert__heading alpha"
                ).text.strip()
                bin_info = {
                    "type": "purple",  # Based on the provided information in the HTML, assuming it's a purple bin day.
                    "collectionDate": self.format_date(next_collection_date),
                }
                bin_data.append(bin_info)

            # Extract data from articles
            articles = soup.find_all("article", class_="binday")
            for article in articles:
                bin_info = self.extract_bin_data(article)
                bin_data.append(bin_info)

            result = {"bins": bin_data}

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return result



import time

from bs4 import BeautifulSoup
from dateutil.relativedelta import relativedelta
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import Select

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://www.midandeastantrim.gov.uk/resident/waste-recycling/collection-dates/"

            # Assign user info
            user_postcode = kwargs.get("postcode")
            # not used: user_paon = kwargs.get("paon")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            # Create Selenium webdriver
            options = webdriver.ChromeOptions()
            options.add_experimental_option("excludeSwitches", ["enable-logging"])
            driver = create_webdriver(web_driver, headless, None, __name__)

            driver.get(page)

            time.sleep(5)
            number = 0
            driver.switch_to.frame(number)
            # Enter postcode in text box and wait
            inputElement_pc = driver.find_element(By.ID, "txtAjaxSearch")
            inputElement_pc.send_keys(user_postcode)

            time.sleep(5)

            # Submit address information and wait - selecting the top one only
            # if it is an exact match then it will go straight to the results
            try:
                button = driver.find_element(By.XPATH, '//*[@id="show-button-0"]')
                driver.execute_script("arguments[0].click();", button)
            except NoSuchElementException:
                pass

            time.sleep(4)

            # Read next collection information
            page = driver.find_element(By.ID, "divCalendarGraphics").get_attribute(
                "outerHTML"
            )

            # Make a BS4 object - remove bold tags and add @ so we can split the lines later
            soup = BeautifulSoup(
                page.strip()
                .replace("<b>", "")
                .replace("</b>", "")
                .replace("<br>", "@"),
                features="html.parser",
            )
            soup.prettify()

            # Data to return
            data = {"bins": []}

            # Valid bin types
            binTypes = ["Refuse", "Garden"]

            # Value to create dict for bin values
            keys, values = [], []

            # Loop though html for text containing bins
            # example of html (bold tags removed above)
            # <div id="divCalendarGraphics">
            # <br>  <b>Refuse</b>: Tue 14 Nov then every alternate  Tue<br><b>Recycling</b>: No Recycling waste collection for this address<br><b>Garden</b>: Tue 21 Nov then every alternate  Tue<br><img src="img/Gif-Spacer.gif" alt="spacer" height="1" width="30">
            # split by br tag and take first 4 splits
            lines = soup.text.split("@", 4)
            for line in lines[1:4]:
                keys.append(line.split(":")[0].strip())
                # strip out the day and month from the text
                values.append(line.split(":")[1].strip().split(" ")[:3])

            # Create dict for bin name and string dates
            binDict = dict(zip(keys, values))

            # Process dict for valid bin types
            for bin in list(binDict):
                if bin in binTypes:
                    # Convert date - no year value so take it from todays date
                    if binDict[bin][0] == "Tomorrow":
                        date = datetime.today() + relativedelta(days=1)
                    elif binDict[bin][0] == "Today":
                        date = datetime.today()
                    else:
                        date = datetime.strptime(
                            " ".join(binDict[bin][1:]), "%d %b"
                        ).replace(year=datetime.today().year)
                        # if the date is in the past then it means the collection is next year so add a year
                        if date < datetime.today():
                            date = date + relativedelta(years=1)

                    # Set bin data
                    dict_data = {
                        "type": bin,
                        "collectionDate": date.strftime(date_format),
                    }
                    data["bins"].append(dict_data)

        # Quit Selenium webdriver to release session
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import time
from datetime import datetime

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://waste.bexley.gov.uk/waste"

            data = {"bins": []}

            user_uprn = kwargs.get("uprn")
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            wait = WebDriverWait(driver, 10)

            inputElement_postcodesearch = wait.until(
                EC.element_to_be_clickable((By.ID, "pc"))
            )
            inputElement_postcodesearch.send_keys(user_postcode)



            find_address_btn = wait.until(
                EC.element_to_be_clickable((By.XPATH, '//*[@id="sub"]'))
            )
            find_address_btn.click()

            dropdown_options = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, '//*[@id="address"]')
                )
            )
            time.sleep(2)
            dropdown_options.click()
            time.sleep(1)

            # Wait for the element to be clickable
            address = WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable(
                    (By.XPATH, f'//li[contains(text(), "{user_paon}")]')
                )
            )

            # Click the element
            address.click()


            submit_address = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, '//*[@id="go"]')
                )
            )
            time.sleep(2)
            submit_address.click()

            results_found = wait.until(
                EC.element_to_be_clickable(
                    (By.XPATH, '//h1[contains(text(), "Your bin days")]')
                )           
                )

            final_page = wait.until(
                EC.presence_of_element_located(
                    (By.CLASS_NAME, "waste__collections")
                )
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            # Find all waste services

            # Initialize the data dictionary
            data = {"bins": []}
            bin_sections = soup.find_all("h3", class_="waste-service-name")

            # Loop through each bin field
            for bin_section in bin_sections:
                # Extract the bin type (e.g., "Brown Caddy", "Green Wheelie Bin", etc.)
                bin_type = bin_section.get_text(strip=True).split("\n")[0]  # The first part is the bin type

                # Find the next sibling <dl> tag that contains the next collection information
                summary_list = bin_section.find_next("dl", class_="govuk-summary-list")

                if summary_list:
                    # Now, instead of finding by class, we'll search by text within the dt element
                    next_collection_dt = summary_list.find("dt", string=lambda text: "Next collection" in text)

                    if next_collection_dt:
                        # Find the sibling <dd> tag for the collection date
                        next_collection = next_collection_dt.find_next_sibling("dd").get_text(strip=True)

                        if next_collection:
                            try:
                                # Parse the next collection date (assuming the format is like "Tuesday 15 October 2024")
                                parsed_date = datetime.strptime(next_collection, "%A %d %B %Y")

                                # Add the bin information to the data dictionary
                                data["bins"].append({
                                    "type": bin_type,
                                    "collectionDate": parsed_date.strftime(date_format),
                                })
                            except ValueError as e:
                                print(f"Error parsing date for {bin_type}: {e}")
                        else:
                            print(f"No next collection date found for {bin_type}")
                    else:
                        print(f"No 'Next collection' text found for {bin_type}")
                else:
                    print(f"No summary list found for {bin_type}")

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import json
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        api_url = "https://www.ealing.gov.uk/site/custom_scripts/WasteCollectionWS/home/FindCollection"
        user_uprn = kwargs.get("uprn")

        # Check the UPRN is valid
        check_uprn(user_uprn)

        # Create the form data
        form_data = {
            "UPRN": user_uprn,
        }

        # Make a request to the API
        requests.packages.urllib3.disable_warnings()
        response = requests.post(api_url, data=form_data)

        json_data = json.loads(response.text)

        data = {"bins": []}

        for param in json_data["param2"]:
            data["bins"].append(
                {
                    "type": param["Service"],
                    "collectionDate": datetime.strptime(
                        param["collectionDateString"], "%d/%m/%Y"
                    ).strftime(date_format),
                }
            )

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

#Cheshire East
class CouncilClass(AbstractGetBinDataClass):
    def parse_data(self, page: str, **kwargs) -> dict:
        soup = BeautifulSoup(page.text, features="html.parser")

        bin_data_dict = {"bins": []}

        table = soup.find("table", {"class": "job-details"})
        if table:
            rows = table.find_all("tr", {"class": "data-row"})

            for row in rows:
                cells = row.find_all(
                    "td", {"class": lambda L: L and L.startswith("visible-cell")}
                )
                labels = cells[0].find_all("label") if cells else []

                if len(labels) >= 3:
                    bin_type = labels[2].get_text(strip=True)
                    collection_date = labels[1].get_text(strip=True)

                    bin_data_dict["bins"].append(
                        {
                            "type": bin_type,
                            "collectionDate": collection_date,
                        }
                    )

        return bin_data_dict



import re
from datetime import datetime

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

class CouncilClass(AbstractGetBinDataClass):
    def parse_data(self, page: str, **kwargs) -> dict:

        user_postcode = kwargs.get("postcode")
        check_postcode(user_postcode)

        # Fetch the page content
        root_url = "https://myproperty.molevalley.gov.uk/molevalley/api/live_addresses/{}?format=json".format(user_postcode)
        response = requests.get(root_url, verify=False)

        if not response.ok:
            raise ValueError("Invalid server response code retrieving data.")

        jsonData = response.json()

        if len(jsonData["results"]["features"]) == 0:
            raise ValueError("No collection data found for postcode provided.")

        properties_found = jsonData["results"]["features"]

        html_data = None
        uprn = kwargs.get("uprn")
        if uprn:
            check_uprn(uprn)
            for item in properties_found:
                if uprn == str(int(item["properties"]["blpu_uprn"])):
                    html_data = item["properties"]["three_column_layout_html"]
                    break
            if html_data is None:
                raise ValueError("No collection data found for UPRN provided.")
        else:
            html_data = properties_found[0]["properties"]["three_column_layout_html"]

        # Conditionally replace the commented-out sections (<!-- ... -->)
        if "<!--" in html_data and "-->" in html_data:
            print("Commented-out section found, replacing comments...")
            html_data = html_data.replace("<!--", "").replace("-->", "")
        else:
            print("No commented-out section found, processing as is.")

        # Process the updated HTML data with BeautifulSoup
        soup = BeautifulSoup(html_data, "html.parser")

        data = {"bins": []}
        all_collection_dates = []
        regex_date = re.compile(r"(\d{2}/\d{2}/\d{4})")  # Adjusted date regex
        regex_additional_collection = re.compile(r"We also collect (.*) on (.*) -")

        # Debugging to verify the HTML content is parsed correctly
        print("HTML content parsed successfully.")

        # Search for the 'Bins and Recycling' panel
        bins_panel = soup.find("h2", string="Bins and Recycling")
        if bins_panel:
            panel = bins_panel.find_parent("div", class_="panel")
            print("Found 'Bins and Recycling' panel.")
            
            # Extract bin collection info from the un-commented HTML
            for strong_tag in panel.find_all("strong"):
                bin_type = strong_tag.text.strip()
                collection_string = strong_tag.find_next("p").text.strip()
                
                # Debugging output
                print(f"Processing bin type: {bin_type}")
                print(f"Collection string: {collection_string}")
                
                match = regex_date.search(collection_string)
                if match:
                    collection_date = datetime.strptime(match.group(1), "%d/%m/%Y").date()
                    data["bins"].append({
                        "type": bin_type,
                        "collectionDate": collection_date.strftime("%d/%m/%Y"),
                    })
                    all_collection_dates.append(collection_date)
                else:
                    # Add a debug line to show which collections are missing dates
                    print(f"No valid date found for bin type: {bin_type}")

            # Search for additional collections like electrical and textiles
            for p in panel.find_all("p"):
                additional_match = regex_additional_collection.match(p.text.strip())
                
                # Debugging output for additional collections
                if additional_match:
                    bin_type = additional_match.group(1)
                    print(f"Found additional collection: {bin_type}")
                    if "each collection day" in additional_match.group(2):
                        if all_collection_dates:
                            collection_date = min(all_collection_dates)
                            data["bins"].append({
                                "type": bin_type,
                                "collectionDate": collection_date.strftime("%d/%m/%Y"),
                            })
                        else:
                            print("No collection dates available for additional collection.")
                            raise ValueError("No valid bin collection dates found.")
                else:
                    print(f"No additional collection found in paragraph: {p.text.strip()}")
        else:
            raise ValueError("Unable to find 'Bins and Recycling' panel in the HTML data.")

        # Debugging to check collected data
        print(f"Collected bin data: {data}")
        
        # Handle the case where no collection dates were found
        if not all_collection_dates:
            raise ValueError("No valid collection dates were found in the data.")
        
        return data


from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Parse the HTML content using BeautifulSoup
        soup = BeautifulSoup(page.text, features="html.parser")

        # Initialize a dictionary to store the parsed bin data
        data = {"bins": []}

        # Define a mapping of bin collection labels to their corresponding types
        bin_types = {
            "Next recycling collection": "Recycling",
            "Next grey bin collection": "Grey Bin",
            "Next brown bin collection": "Brown Bin",
            "Next food bin collection": "Food Bin",
        }

        # Locate the <ul> element with the class "data-table"
        bin_collections = soup.find("ul", {"class": "data-table"})

        # Proceed only if the <ul> element is found
        if bin_collections:
            # Retrieve all <li> elements within the <ul>, skipping the first two (not relevant)
            bin_items = bin_collections.find_all("li")[2:]

            # Iterate through each bin item
            for bin in bin_items:
                bin_type = None
                # Retrieve the bin type from the header if it exists
                if bin.h2 and bin.h2.text.strip() in bin_types:
                    bin_type = bin_types[bin.h2.text.strip()]

                bin_collection_date = None
                # Retrieve the bin collection date from the div if it exists
                if bin.div and bin.div.text.strip():
                    try:
                        # Parse the collection date from the div text and format it
                        bin_collection_date = datetime.strptime(
                            bin.div.text.strip(),
                            "%A %d/%m/%Y",
                        ).strftime(date_format)
                    except ValueError:
                        # If date parsing fails, keep bin_collection_date as None
                        pass

                # If both bin type and collection date are identified, add to the data
                if bin_type and bin_collection_date:
                    data["bins"].append(
                        {
                            "type": bin_type,
                            "collectionDate": bin_collection_date,
                        }
                    )

        # Return the parsed data, which may be empty if no bins were found
        return data



import time

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def extract_styles(self, style_str: str) -> dict:
        return dict(
            (a.strip(), b.strip())
            for a, b in (
                element.split(":") for element in style_str.split(";") if element
            )
        )

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://www.northumberland.gov.uk/Waste/Bins/Bin-Calendars.aspx"

            data = {"bins": []}

            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_paon(user_paon)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            time.sleep(1)

            # Press the cookie accept - wait is to let the JS load it up
            driver.find_element(By.ID, "ccc-notify-accept").click()

            inputElement_hn = driver.find_element(
                By.ID,
                "p_lt_ctl04_pageplaceholder_p_lt_ctl02_WasteCollectionCalendars_NCCAddressLookup_txtHouse",
            )
            inputElement_pc = driver.find_element(
                By.ID,
                "p_lt_ctl04_pageplaceholder_p_lt_ctl02_WasteCollectionCalendars_NCCAddressLookup_txtPostcode",
            )

            inputElement_pc.send_keys(user_postcode)
            inputElement_hn.send_keys(user_paon)

            driver.find_element(
                By.ID,
                "p_lt_ctl04_pageplaceholder_p_lt_ctl02_WasteCollectionCalendars_NCCAddressLookup_butLookup",
            ).click()

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            # Work out which bins can be collected for this address. Glass bins are only on some houses due to pilot programme.
            bins_collected = list(
                map(
                    str.strip,
                    soup.find(
                        "span",
                        id="p_lt_ctl04_pageplaceholder_p_lt_ctl02_WasteCollectionCalendars_spanRouteSummary",
                    )
                    .string.replace("Routes found: ", "")
                    .split(","),
                )
            )

            # Get the background colour for each of them...
            bins_by_colours = dict()
            for bin in bins_collected:
                style_str = soup.find("span", string=bin)["style"]
                bin_colour = self.extract_styles(style_str)["background-color"].upper()
                bins_by_colours[bin_colour] = bin

            # Work through the tables gathering the dates, if the cell has a background colour - match it to the bin type.
            calander_tables = soup.find_all("table", title="Calendar")
            for table in calander_tables:
                # Get month and year
                # First row in table is the header
                rows = table.find_all("tr")
                month_and_year = (
                    rows[0].find("table", class_="calCtrlTitle").find("td").string
                )
                bin_days = table.find_all("td", class_="calCtrlDay")
                for day in bin_days:
                    day_styles = self.extract_styles(day["style"])
                    if "background-color" in day_styles:
                        colour = day_styles["background-color"].upper()
                        date = time.strptime(
                            f"{day.string} {month_and_year}", "%d %B %Y"
                        )

                        # Add it to the data
                        data["bins"].append(
                            {
                                "type": bins_by_colours[colour],
                                "collectionDate": time.strftime(date_format, date),
                            }
                        )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Get postcode and UPRN from kwargs
        user_postcode = kwargs.get("postcode")
        user_paon = kwargs.get("paon")
        web_driver = kwargs.get("web_driver")
        headless = kwargs.get("headless")
        check_postcode(user_postcode)
        check_paon(user_paon)

        # Build URL to parse
        council_url = "https://swale.gov.uk/bins-littering-and-the-environment/bins/my-collection-day"

        # Create Selenium webdriver
        driver = create_webdriver(web_driver, headless, None, __name__)
        driver.get(council_url)

        # Wait for the postcode field to appear then populate it
        try:
            inputElement_postcode = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located((By.ID, "q462406_q1"))
            )
            inputElement_postcode.send_keys(user_postcode)
        except Exception:
            print("Page failed to load. Probably due to Cloudflare robot check!")

        # Click search button
        findAddress = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.ID, "form_email_462397_submit"))
        )
        driver.execute_script("arguments[0].click();", findAddress)

        # Wait for the 'Select address' dropdown to appear and select option matching the house name/number
        WebDriverWait(driver, 10).until(
            EC.element_to_be_clickable(
                (
                    By.XPATH,
                    "//select[@id='SBCYBDAddressList']//option[contains(., '"
                    + user_paon
                    + "')]",
                )
            )
        ).click()

        # Click search button
        getBins = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.ID, "form_email_462397_submit"))
        )
        driver.execute_script("arguments[0].click();", getBins)

        BinTable = WebDriverWait(driver, 30).until(
            EC.presence_of_element_located((By.ID, "SBC-YBD-Main"))
        )

        soup = BeautifulSoup(driver.page_source, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        # Get the collection bullet points on the page and parse them
        nextCollections = soup.find("div", {"id": "nextCollections"})
        for c in nextCollections:
            collection = c.find_all("strong")
            for bin in collection:
                split = (bin.text).split(" on ")
                bin_type = split[0]
                bin_date = datetime.strptime(split[1], "%A %d %b %Y").strftime(
                    "%d/%m/%Y"
                )
                dict_data = {"type": bin_type, "collectionDate": bin_date}
                data["bins"].append(dict_data)

        return data
